/**
  ******************************************************************************
  * @file    MainH.C
  * @author  Muhammad Shahzad
  * @version V1.0
  * @date    24-December-2014
  * @brief   in this proejct get the time from RTC and display on GLCD
  * @email   itzshahzad@gmail.com
  ******************************************************************************
  ******************************************************************************
  */

char time_str[15];
char date_str[18];
// Glcd module connections
char GLCD_DataPort at PORTD;
sbit GLCD_CS1 at RB2_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_CS2 at RB1_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RS  at RB0_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RW  at RE2_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_EN  at RE1_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RST at RE0_bit;  // for writing to output pin always use latch (PIC18 family)

sbit GLCD_CS1_Direction at TRISB2_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB0_bit;
sbit GLCD_RW_Direction  at TRISE2_bit;
sbit GLCD_EN_Direction  at TRISE1_bit;
sbit GLCD_RST_Direction at TRISE0_bit;
// End Glcd module connections

unsigned char sec, min1, hr, week_day, day, mn, year;
char *txt, tnum[4];

//--------------------- Reads time and date information from RTC (DS1307)
void Read_Time(char *sec, char *min, char *hr, char *week_day, char *day, char *mn, char *year) {
  I2C1_Start();
  I2C1_Wr(0xD0);
  I2C1_Wr(0);
  I2C1_Repeated_Start();
  I2C1_Wr(0xD1);
  *sec =I2C1_Rd(1);
  *min =I2C1_Rd(1);
  *hr =I2C1_Rd(1);
  *week_day =I2C1_Rd(1);
  *day =I2C1_Rd(1);
  *mn =I2C1_Rd(1);
  *year =I2C1_Rd(0);
  I2C1_Stop();
}//~

//-------------------- Formats date and time
void Transform_Time(char  *sec, char *min, char *hr, char *week_day, char *day, char *mn, char *year) {
  *sec  =  ((*sec & 0x70) >> 4)*10 + (*sec & 0x0F);
  *min  =  ((*min & 0xF0) >> 4)*10 + (*min & 0x0F);
  *hr   =  ((*hr & 0x30) >> 4)*10 + (*hr & 0x0F);
  *week_day =(*week_day & 0x07);
  *day  =  ((*day & 0xF0) >> 4)*10 + (*day & 0x0F);
  *mn   =  ((*mn & 0x10) >> 4)*10 + (*mn & 0x0F);
  *year =  ((*year & 0xF0)>>4)*10+(*year & 0x0F);
}//~

//-------------------- Output values to LCD
void Display_Time(char sec, char min, char hr, char week_day, char day, char mn, char year) {

   //LCD_Out(1,1,txt);
   date_str[5] = ((day / 10) + 48   );    // Print tens digit of day variable
   date_str[6] = ((day % 10) + 48);    // Print oness digit of day variable
   date_str[7] = (':');
   date_str[8] = ( (mn / 10) + 48);
   date_str[9] = ( (mn % 10) + 48);
   date_str[10] = ( ':');
   date_str[11] = ( '2');
   date_str[12] = ( '0');
   date_str[13] = ( (year / 10) + 48);          // Print year vaiable + 8 (start from year 2008)
   date_str[14] = ( (year % 10) + 48);          // Print year vaiable + 8 (start from year 2008)
   date_str [15] =0 ;
   
   time_str[5] = ( (hr / 10)  + 48 );
   time_str[6] = ( (hr % 10)  + 48 );
   time_str[7] = ( ':');
   time_str[8] = ( (min / 10) + 48 );
   time_str[9] = ( (min % 10) + 48 );
   time_str[10] = ( ':');
   time_str[11] = ( (sec / 10) + 48 );
   time_str[12] = ( (sec % 10) + 48 );
   time_str[13] = 0;
}//~

//------------------ Performs project-wide init
void Init_Main() {

  I2C1_Init(400000);                        // initialize I2C


}//~


void main() {

   PCFG3_bit=1; PCFG2_bit=1; PCFG1_bit=1; PCFG0_bit=1;
   TRISE2_bit=0;TRISE1_bit=0;//Analog Mux as out put Initlization;
   Glcd_Init();                              // Initialize GLCD
   
   UART1_Init(9600);               // Initialize UART module at 9600 bps
   Delay_ms(100);                  // Wait for UART module to stabilize

  UART1_Write_Text("Main Start\r");

   Glcd_Fill(0x00);                          // Clear GLCD
   Glcd_Set_Font(font5x7, 5, 7, 32);    Glcd_Write_Text("Hello world!", 10, 2, 1);
   Init_Main();
   
   time_str[0]='T';time_str[1]='i';time_str[2]='m';time_str[3]='e';time_str[4]=':';
   date_str[0]='D';date_str[1]='a';date_str[2]='t';date_str[3]='e';date_str[4]=':';
  
  while(1) {

    //Glcd_Write_Text("Time and Date", 23, 0, 2);
    Read_Time(&sec,&min1,&hr,&week_day,&day,&mn,&year);      // read time from RTC(DS1307)
    Transform_Time(&sec,&min1,&hr,&week_day,&day,&mn,&year); // format date and time
    Display_Time(sec, min1, hr, week_day, day, mn, year);    // prepare and display on LCD
     
      Glcd_Write_Text(time_str, 2, 2, 1);
      Glcd_Write_Text(date_str, 2, 4, 1);

      UART1_Write_Text(time_str);UART1_Write_Text("\r");
      UART1_Write_Text(date_str);UART1_Write_Text("\r\r");
      Delay_ms(1000);

  }
}